<?php

namespace Drupal\group_behavior_domain_sync;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\group\Entity\GroupContent;
use Drupal\group\Entity\GroupContentInterface;
use Drupal\group\Entity\GroupContentTypeInterface;
use Drupal\group_behavior\GroupBehaviorHelpers;

class GroupBehaviorDomainSyncEntityHooks {

  protected static $doingDomainSync = FALSE;

  /**
   * Entity postSave = insert + update.
   *
   * If we were using the OG module, we could do this in presave as we there
   * have a group reference field. But we are in group.
   *
   * For entities which are updated without changing their GroupContent
   * relations, everything is fine.
   *
   * The computed field in https://www.drupal.org/project/group/issues/2813405
   * updates GroupContent relations in FieldItemList::postSave, which runs
   * before this entity postsave. All fine.
   *
   * For entities created via the GroupContent form, this is more complicated.
   * I currently could not find the source, but the entity must first be saved
   * so it has an ID and only then the GroupContent is saved.
   * After that, in GroupContent::postSave, the entity is saved again to update
   * some caches.
   *
   * Fortunately (erh hopefully if i got things right) this means, we can safely
   * rely on the GroupContent relation when this is called. (And ignore the
   * missing relation on the first save.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   */
  public static function postSave(EntityInterface $entity) {
    if (static::$doingDomainSync) {
      return;
    }
    static::$doingDomainSync = TRUE;
    self::doPostSave($entity);
    static::$doingDomainSync = FALSE;
  }

  /**
   * @param \Drupal\Core\Entity\EntityInterface $entity
   */
  protected static function doPostSave(EntityInterface $entity) {
    if ($entity instanceof ContentEntityInterface && static::hasDomainField($entity)) {
      if ($groupContents = GroupContent::loadByEntity($entity)) {
        $syncedToThis = FALSE;
        /** @var \Drupal\group\Entity\GroupContentInterface[] $groupContents */
        foreach ($groupContents as $groupContent) {
          $groupContentType = $groupContent->getGroupContentType();
          $hasDomainSync = self::hasDomainSync($groupContentType);
          if ($hasDomainSync) {
            $hasGroupBehavior = GroupBehaviorHelpers::checkGroupContentTypeHasGroupBehavior($groupContentType);
            if ($hasGroupBehavior) {
              self::syncDomainFieldsToOtherGroupContent($groupContent);
            }
            else {
              if ($syncedToThis) {
                throw new \LogicException("Trying to sync twice to {$entity->getEntityTypeId()} {$entity->id()}");
              }
              self::syncDomainFieldsToThisGroupContent($groupContent);
              $syncedToThis = TRUE;
            }
          }
        }
      }
    }
  }

  protected static function syncDomainFieldsToOtherGroupContent(GroupContentInterface $groupBehaviorContent) {
    // Sync domain fields to all other group content with domain sync enabled type.
    // Caution: This will break for lots of content.
    // @todo Make this scale well.
    // @todo Only do this when domain field data changed.
    /** @var FieldableEntityInterface $groupBehaviorEntity */
    $groupBehaviorEntity = $groupBehaviorContent->getEntity();
    $group = $groupBehaviorContent->getGroup();
    $groupId = $group->id();
    $groupTypeId = $group->getGroupType()->id();
    // As groupContent has group_behavior true, we know it is not in this
    // result set.
    $groupContentTypeIds = \Drupal::entityQuery('group_content_type')
      ->condition('group_type', $groupTypeId)
      ->condition('third_party_settings.group_behavior.domain_sync', TRUE)
      ->condition('third_party_settings.group_behavior.enabled', TRUE, '<>')
      ->execute();
    if ($groupContentTypeIds) {
      $groupContentIds = \Drupal::entityQuery('group_content')
        ->condition('gid', $groupId)
        ->condition('type', $groupContentTypeIds)
        ->execute();
      $groupContents = GroupContent::loadMultiple($groupContentIds);
      foreach ($groupContents as $groupContent) {
        /** @var ContentEntityInterface $groupContentEntity */
        $groupContentEntity = $groupContent->getEntity();
        foreach (static::domainFieldNames() as $fieldName) {
          $value = $groupBehaviorEntity->get($fieldName)->getValue();
          $groupContentEntity->set($fieldName, $value);
        }
        $groupContentEntity->save();
      }
    }
  }

  protected static function syncDomainFieldsToThisGroupContent(GroupContentInterface $groupContent) {
    /** @var FieldableEntityInterface $entity */
    $entity = $groupContent->getEntity();

    $group = $groupContent->getGroup();
    $groupId = $group->id();
    $groupTypeId = $group->getGroupType()->id();
    // As groupContent has group_behavior false, we know it is not in this
    // result set.
    $groupBehaviorContentTypeIds = \Drupal::entityQuery('group_content_type')
      ->condition('group_type', $groupTypeId)
      ->condition('third_party_settings.group_behavior.domain_sync', TRUE)
      ->condition('third_party_settings.group_behavior.enabled', TRUE)
      ->execute();
    if ($groupBehaviorContentTypeIds) {
      $groupBehaviorContentIds = \Drupal::entityQuery('group_content')
        ->condition('gid', $groupId)
        ->condition('type', $groupBehaviorContentTypeIds)
        ->execute();
      $groupBehaviorContents = GroupContent::loadMultiple($groupBehaviorContentIds);

      if (count($groupBehaviorContents) > 1) {
        $groupBehaviorContentTypeIds = implode('|', array_keys($groupBehaviorContents));
        throw new \LogicException("Group behavior not unique for {$entity->getEntityTypeId()} {$entity->id()}: $groupBehaviorContentTypeIds");
      }
      if ($groupBehaviorContents) {
        $groupBehaviorContent = reset($groupBehaviorContents);
        /** @var ContentEntityInterface $groupBehaviorEntity */
        $groupBehaviorEntity = $groupBehaviorContent->getEntity();
        foreach (static::domainFieldNames() as $domainFieldName) {
          $value = $groupBehaviorEntity->get($domainFieldName)->getValue();
          $entity->set($domainFieldName, $value);
        }
        $entity->save();
      }
    }
  }

  protected static function hasDomainField(EntityInterface $entity){
    return $entity instanceof FieldableEntityInterface
      && array_intersect(
        array_keys($entity->getFieldDefinitions()),
        static::domainFieldNames()
      );
  }

  protected static function domainFieldNames() {
    return [DOMAIN_ACCESS_FIELD, DOMAIN_ACCESS_ALL_FIELD];
  }

  /**
   * @param \Drupal\group\Entity\GroupContentTypeInterface $groupContentType
   *
   * @return mixed
   */
  protected static function hasDomainSync(GroupContentTypeInterface $groupContentType) {
    return GroupBehaviorHelpers::checkContentTypeSetting($groupContentType, 'domain_sync');
  }
}
