<?php

namespace Drupal\group_behavior;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\group\Entity\GroupContent;
use Drupal\group\Entity\GroupContentInterface;
use Drupal\group\Entity\GroupContentType;
use Drupal\group\Entity\GroupContentTypeInterface;

class GroupBehaviorHelpers {

  /**
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *
   * @return array
   */
  public static function fetchGroupBehaviorGroupsOfEntity(EntityInterface $entity) {
    $groups = [];
    if ($entity instanceof ContentEntityInterface) {
      if ($groupContents = GroupContent::loadByEntity($entity)) {
        /** @var \Drupal\group\Entity\GroupContentInterface[] $groupContents */
        foreach ($groupContents as $groupContent) {
          $groupContentType = $groupContent->getGroupContentType();
          if (static::checkGroupContentTypeHasGroupBehavior($groupContentType)) {
            $groups[] = $groupContent->getGroup();
          }
        }
      }
    }
    return $groups;
  }

  /**
   * Filter group content by type.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   * @param \Drupal\group\Entity\GroupContentTypeInterface $groupContentType
   *
   * @return \Drupal\group\Entity\GroupContentInterface[]
   */
  public static function fetchEntityGroupContentOfType(EntityInterface $entity, GroupContentTypeInterface $groupContentType) {
    if ($entity instanceof ContentEntityInterface) {
      $entityGroupContents = GroupContent::loadByEntity($entity);
      $groupContentTypeId = $groupContentType->id();
      return array_filter($entityGroupContents, function (GroupContentInterface $groupContent) use ($groupContentTypeId) {
        return $groupContent->getGroupContentType()->id() === $groupContentTypeId;
      });
    }
    else {
      return [];
    }
  }

  /**
   * Get applicable group content types.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity to check.
   *
   * @return \Drupal\group\Entity\GroupContentTypeInterface[]
   *   Group content types.
   */
  public static function fetchGroupContentTypesWithGroupBehaviorByEntity(EntityInterface $entity) {
    /** @var \Drupal\group\Entity\GroupContentTypeInterface[] $groupContentTypes */
    $groupContentTypes = GroupContentType::loadByEntityTypeId($entity->getEntityTypeId());
    $groupContentTypes = self::filterGroupContentTypesWithGroupBehavior($groupContentTypes);
    $groupContentTypes = self::filterGroupContentTypesByBundle($groupContentTypes, $entity->bundle());
    return $groupContentTypes;
  }

  /**
   * Filter group content types by setting.
   *
   * @param \Drupal\group\Entity\GroupContentTypeInterface[] $groupContentTypes
   *   Group content types.
   *
   * @return \Drupal\group\Entity\GroupContentTypeInterface[]
   *   Group content types.
   */
  protected static function filterGroupContentTypesWithGroupBehavior($groupContentTypes) {
    $applicableGroupContentTypes = [];
    foreach ($groupContentTypes as $id => $groupContentType) {
      if (self::checkGroupContentTypeHasGroupBehavior($groupContentType)) {
        $applicableGroupContentTypes[$id] = $groupContentType;
      }
    }
    return $applicableGroupContentTypes;
  }

  /**
   * @param \Drupal\group\Entity\GroupContentTypeInterface[] $groupContentTypes
   *   Group content types.
   * @param string $bundle
   *   The bundle.
   * @return \Drupal\group\Entity\GroupContentTypeInterface[]
   *   Group content types.
   */
  protected static function filterGroupContentTypesByBundle($groupContentTypes, $bundle) {
    $applicableGroupContentTypes = [];
    foreach ($groupContentTypes as $id => $groupContentType) {
      if ($bundle === $groupContentType->getContentPlugin()->getEntityBundle()) {
        $applicableGroupContentTypes[$id] = $groupContentType;
      }
    }
    return $applicableGroupContentTypes;
  }

  /**
   * @param \Drupal\group\Entity\GroupContentTypeInterface $groupContentType
   *
   * @return boolean
   */
  public static function checkGroupContentTypeHasGroupBehavior(GroupContentTypeInterface $groupContentType) {
    return self::checkContentTypeSetting($groupContentType, 'enabled');
  }

  /**
   * @param \Drupal\group\Entity\GroupContentTypeInterface $groupContentType
   * @param $setting
   *
   * @return mixed
   */
  public static function checkContentTypeSetting(GroupContentTypeInterface $groupContentType, $setting) {
    return $groupContentType->getThirdPartySetting('group_behavior', $setting);
  }


}
